#! /usr/bin/python3

import argparse
import json
import sys


def deepmerge_dict(data_a, data_b, *, path):
    result ={}
    for key in data_a:
        if key not in data_b:
            result[key] = data_a[key]
        else:
            result[key] = deepmerge(data_a[key], data_b[key], path=path + "." + key)
    for key in data_b:
        if key not in data_a:
            result[key] = data_b[key]
    return result


def deepmerge(data_a, data_b, *, path=""):
    if type(data_a) is not type(data_b):
        raise TypeError('mismatched types at %s' % path)
    if isinstance(data_a, dict):
        return deepmerge_dict(data_a, data_b, path=path)
    if isinstance(data_a, list):
        return data_a + data_b
    if data_a != data_b:
        raise ValueError('mismatched values at %s' % path)
    


def main():
    """Main function."""
    parser = argparse.ArgumentParser("Merge fragments of manifest.json files.")
    parser.add_argument('--merge', nargs='+', help='input files to merge')
    parser.add_argument('--override', nargs='*', default=[], help='override files')
    opts = parser.parse_args()

    result = {}
    errors = False
    for path in opts.merge:
        with open(path, 'r') as fp:
            try:
                data = json.load(fp)
            except json.decoder.JSONDecodeError as exc:
                print('%s: %s' % (path, str(exc)), file=sys.stderr)
                errors = True
        if not errors:
            result = deepmerge(result, data)

    if errors:
        sys.exit(1)

    for path in opts.override:
        with open(path, 'r') as fp:
            try:
                data = json.load(fp)
            except json.decoder.JSONDecodeError as exc:
                print('%s: %s' % (path, str(exc)), file=sys.stderr)
                errors = True
        if not errors:
            result.update(data)
        
    print(json.dumps(result, indent=4))


if __name__ == '__main__':
    main()
